type primitive = null | undefined | string | symbol | number | boolean | bigint;

declare function ToPrimitive(input: unknown, hint?: StringConstructor | NumberConstructor): primitive;

export = ToPrimitive;
